#ifndef _FASTSET_H_
#define _FASTSET_H_

#include "string.h"

template <typename IntType = short>
class FastSet {
public:
  FastSet(size_t aSize = 0) 
  : data(0), 
    size(0), 
    mark(0) 
  {
    setSize(aSize);
  }
  ~FastSet() { 
    delete[] data; 
  }
  void setSize(size_t aSize) {
    if (aSize != size) {
      delete[] data;
      data = new IntType[aSize];
      size = aSize;
      mark = -1;
    }
    reset();
  }
  int getSize() const { 
    return size; 
  }
  void reset() {
    if (++mark == 0) {
      memset(data, 0, size * sizeof(data[0]));
    }
  }
  bool get(size_t aPos) const {
    return data[aPos] == mark;
  }
  void set(size_t aPos) {
    data[aPos] = mark;
  }
  void set(size_t aPos, bool aValue) {
    data[aPos] = aValue ? mark : (mark - 1);
  }
  bool check(size_t aPos) { // Set and return previous state.
    if (data[aPos] == mark) return true;
    data[aPos] = mark;
    return false;
  }
  bool check(size_t aPos, bool aValue) {
    IntType old = data[aPos];
    data[aPos] = aValue ? mark : (mark - 1);
    return (old == mark);
  }
private:
  IntType* data;
  size_t size;
  IntType mark;
  FastSet(const FastSet&);
  FastSet& operator=(const FastSet&);
};

#endif

