#include <cstring>
#include <climits>

#include "utils.h"
#include "keylist.h"

using namespace std;


void Key::init(
  int aNo,
  const char* aKey,
  int aKeyLen,
  const char* aData)
{
  no = aNo;
  keyLen = aKeyLen;
  key = new byte[aKeyLen + 1];
  memcpy(key, aKey, keyLen);
  key[keyLen] = 0;
  hashKey = sortedHashKey = 0;
  data = strAlloc(aData);
  charSet = 0;
}

void Key::free() {
  delete[] key;
  delete[] hashKey;
  delete[] sortedHashKey;
  delete[] charSet;
  strFree(data);
}

void KeyList::clear() {
  for (Key* key = keys; key != end; key++) {
    key->free();
  }
  delete[] keys;
  init();
}

void KeyList::init() {
  keys = 0;
  end = 0;
  keyAlloc = 0;
  keyCount = 0;
  minKeyLen = INT_MAX;
  maxKeyLen = 0;
}

void KeyList::add(const char* aKey, int aLen, const char* aData) {
  if (keyCount >= keyAlloc) {
    int tempKeyAlloc = (keyAlloc < 256) ? 256 : (keyAlloc << 1);
    Key* tempKeys = new Key[tempKeyAlloc];
    memcpy(tempKeys, keys, keyCount * sizeof(Key));
    delete[] keys;
    keys = tempKeys;
    keyAlloc = tempKeyAlloc;
  }
  keys[keyCount].init(keyCount, aKey, aLen, aData);
  keyCount++;
  end = keys + keyCount;
  if (aLen < minKeyLen) {
    minKeyLen = aLen;
  }
  if (aLen > maxKeyLen) {
    maxKeyLen = aLen;
  }
}

