#include <cstring>
#include <cstdio>
#include "options.h"

using namespace std;


OptionParser::OptionParser(const Option* aOptions, const char* aArgs[]) {
  opts = aOptions;
  args = aArgs;
  reset();
}

void OptionParser::reset() {
  index = 1;
  arg = args[index];
  shortOptions = false;
  endOfOptions = false;
  error = false;
  errorMessage[0] = 0;
}

bool OptionParser::get(const Option*& aOpt, const char*& aArg) {
  for (;;) {
    if (error || (arg == 0)) {
      return false;
    }
    if (shortOptions) {
      return getShort(aOpt, aArg);
    }
    if (!endOfOptions && (*arg == '-')) { // "-*"
      arg++;
      if (*arg == '-') { // "--*"
        arg++;
        if (*arg == 0) { // "--"
          endOfOptions = true;
          arg = args[++index];
        }
        else { // "--x"
          return getLong(aOpt, aArg);
        }
      }
      else if (*arg == 0) { // "-"
        arg--;
        return getNonOption(aOpt, aArg);
      }
      else { // "-x"
        return getShort(aOpt, aArg);
      }
    }
    else { // "x"
      return getNonOption(aOpt, aArg);
    }
  }
}

bool OptionParser::getNonOption(const Option*& aOpt, const char*& aArg) {
  aOpt = find(Option::nonOption);
  if (aOpt == 0) {
    error = true;
    sprintf(errorMessage, "Unexpected non-option argument");
    return false;
  }
  aArg = arg;
  arg = args[++index];
  return true;
}

bool OptionParser::getLong(const Option*& aOpt, const char*& aArg) {
  char name[50];
  size_t len;
  aArg = strchr(arg, '=');
  if (aArg) { // "--name=val"
    len = aArg - arg;
    aArg++;
  }
  else { // "--name"
    len = strlen(arg);
  }
  if (len > sizeof(name) - 1) {
    error = true;
    sprintf(errorMessage, "Option name is too long");
    return false;
  }
  memcpy(name, arg, len);
  name[len] = 0;
  arg = args[++index];
  aOpt = find(name);
  if (aOpt == 0) {
    if ((aArg == 0) && (len > 3) && (strncmp(name, "no-", 3) == 0)) {
      memmove(name, name + 3, sizeof(name) - 3);
      aOpt = find(name);
      if (aOpt && (aOpt->flags & Option::argBoolean)) {
        aArg = "0";	
        return true;
      }
    }
    error = true;
    sprintf(errorMessage, "Unknown option `--%s'", name);
    return false;
  }
  if (aOpt->flags & (Option::argRequired | Option::argBoolean)) {
    if ((aArg == 0) && (aOpt->flags & Option::argBoolean)) {
      aArg = "1";
    }
    if (aArg == 0) {
      error = true;
      sprintf(errorMessage, "Option `--%s' requires an argument", name);
      return false;
    }
  }
  return true;
}

bool OptionParser::getShort(const Option*& aOpt, const char*& aArg) {
  shortOptions = true;
  int id = *arg++;
  aOpt = find(id);
  if (aOpt == 0) {
    error = true;
    sprintf(errorMessage, "Unknown option `-%c'", id);
    return false;
  }
  if (aOpt->flags & Option::argRequired) {
    shortOptions = false;
    if (*arg == 0) {
      arg = args[++index];
      if (arg == 0) {
        error = true;
        sprintf(errorMessage, "Option `-%c' requires an argument", id);
        return false;
      }
      aArg = arg;
    }
    else {
      aArg = arg;
    }
    arg = args[++index];
  }
  else {
    if (aOpt->flags & Option::argBoolean) {
      switch (*arg) {
        case '-': aArg = "0"; arg++; break;
        case '+': aArg = "1"; arg++; break;
        default : aArg = "1"; break;
      }
    }
    else {
      aArg = "";
    }
    if (*arg == 0) {
      shortOptions = false;
      arg = args[++index];
    }
  }
  return true;
}

const Option* OptionParser::find(const char* aName) {
  for (const Option* opt = opts; opt->id != Option::end; opt++) {
    if (opt->name && (strcmp(aName, opt->name) == 0)) return opt;
  }
  return 0;
}


const Option* OptionParser::find(int aId) {
  for (const Option* opt = opts; opt->id != Option::end; opt++) {
    if (aId == opt->id) return opt;
  }
  return 0;
}



