#ifndef _OPTIONS_H_
#define _OPTIONS_H_

struct Option {
  int id;
  const char* name;
  int flags;
  enum { // special id values
    end = 0, // should terminate the list
    nonOption // allows non-option arguments
  };
  enum { // flags
    argRequired = 1,
    argBoolean  = 4
  };
};

class OptionParser {
public:
  OptionParser(const Option* aDefs, const char* aArgs[]);
  void reset();
  bool get(const Option*& aOption, const char*& aArg);
  bool getError() const { return error; }
  const char* getErrorMessage() const { return errorMessage; }
private:
  const Option* opts;
  const char* const* args;
  int index;
  const char* arg;
  bool shortOptions; // inside a cluster of short options
  bool endOfOptions; // after `--'
  bool error;
  char errorMessage[256];
  const Option* find(int);
  const Option* find(const char*);
  bool getShort(const Option*& aOpt, const char*& aArg);
  bool getLong(const Option*& aOpt, const char*& aArg);
  bool getNonOption(const Option*& aOpt, const char*& aArg);
};

#endif

