#include <iostream>
#include <cstdio>
#include <cstdarg>
#include "hashgen.h"

using namespace std;


void HashGenerator::openOutput(const string& aFileName) {
  if (aFileName.empty()) {
    outputStream = &cout;
  }
  else {
    outputFile.open(aFileName.c_str());
    outputStream = &outputFile;
  }
  outputIndent = 0;
  outputColumn = 0;
  linePrefix = "";
}

void HashGenerator::closeOutput() {
  if (outputStream == &outputFile) {
    outputFile.close();
    outputStream = &cout;
  }
}

void HashGenerator::write(const char* aBuf) {
  for ( ; *aBuf; aBuf++) {
    if (outputColumn == 0) {
      for (int i = 0; i < outputIndent; i++) {
        *outputStream << "  ";
      }
      *outputStream << linePrefix;
    }
    *outputStream << *aBuf;
    outputColumn++;
    if (*aBuf == '\n') {
      outputColumn = 0;
    }
  }
}

void HashGenerator::out(const char* aFormat, ...) {
  char buf[writeBufSize];
  va_list arg;
  va_start(arg, aFormat);
  vsprintf(buf, aFormat, arg);
  va_end(arg);
  write(buf);
}

void HashGenerator::log(const char* aFormat, ...) {
  char buf[writeBufSize];
  va_list arg;
  va_start(arg, aFormat);
  vsprintf(buf, aFormat, arg);
  va_end(arg);
  cerr << buf;
}

void HashGenerator::die(const char* aFormat, ...) {
  char buf[writeBufSize];
  va_list arg;
  va_start(arg, aFormat);
  vsprintf(buf, aFormat, arg);
  va_end(arg);
  throw Exception(buf);
}

