#ifndef _RAND_H_
#define _RAND_H_

// Needed a portable and reentrant (just in case) random generator.

class RandomGenerator {
public:
  RandomGenerator(long aSeed = 87654321L) 
  : value(aSeed)
  {} 
  RandomGenerator(const RandomGenerator& aGenerator) 
  : value(aGenerator.value)
  {}
  void set(long aSeed) { value = aSeed; }
  long get(); 
  long operator()() { return get(); }
private:
  long value;
};


// Random: Generates random numbers within a given range.
// Associated with a RandomGenerator, which keeps the sequence state.
// May be used as an STL generator function object.

class Random {
public:
  Random(RandomGenerator& aGenerator, long aModulo)
  : generator(aGenerator),
    modulo(aModulo)
  {}
  long operator()() { return generator() % modulo; }
private:
  RandomGenerator& generator;
  long modulo;
};

#endif

