#include <cstring>
#include <cstdlib>
#include <cctype>
#include <climits>
#include "utils.h"

using namespace std;

int minWidth(unsigned aValue) {
  int result = 1;
  for ( ; aValue >= 10; result++, aValue /= 10) {}
  return result;
}

const char* minType(unsigned aValue) {
  if      (aValue <= UCHAR_MAX) return "unsigned char";
  else if (aValue <= USHRT_MAX) return "unsigned short";
  return "unsigned";
}

int minSize(unsigned aValue) {
  if      (aValue <= UCHAR_MAX) return 1;
  else if (aValue <= USHRT_MAX) return 2;
  return 4;
}

int bitCount(unsigned aValue) {
  static char bits[16] = {
    0, 1, 1, 2, 1, 2, 2, 3,
    1, 2, 2, 3, 2, 3, 3, 4
  };
  int result = 0;
  while (aValue) {
    result += bits[aValue & 15];
    aValue >>= 4;
  }
  return result;
}

char* strAlloc(const char* aStr, int aLen) {
  char* str = new char[aLen + 1];
  memcpy(str, aStr, aLen);
  str[aLen] = 0;
  return str;
}

char* strAlloc(const char* aStr) {
  return strAlloc(aStr, static_cast<int>(strlen(aStr)));
}

void strFree(char* aStr) {
  delete[] aStr;
}

char* strUpper(char* aStr) {
  for (char* str = aStr; *str; str++) {
    *str = toupper((unsigned char)(*str));
  }
  return aStr;
}

char* strTrim(char* aStr) {
  char* str = aStr;
  for ( ; isspace(*str); str++) { }
  char* end = str + strlen(str);
  for ( ; (end != str) && isspace(end[-1]); end--) { }
  *end = 0;
  memmove(aStr, str, end - str + 1);
  return aStr;
}

int strQuote(const char* aData, int aDataSize, char* aStr) {
  static const char octal[] = "01234567";
  char* str = aStr;
  const unsigned char* data = reinterpret_cast<const unsigned char*>(aData);
  *str++ = '"';
  while (aDataSize--) {
    switch (*data) {
      case '\r': *str++ = '\\'; *str++ = 'r'; break;
      case '\n': *str++ = '\\'; *str++ = 'n'; break;
      case '\t': *str++ = '\\'; *str++ = 't'; break;
      case '\v': *str++ = '\\'; *str++ = 'v'; break;
      case '\\': *str++ = '\\'; *str++ = '\\'; break;
      case '\"': *str++ = '\\'; *str++ = '"'; break;
      case '\b': *str++ = '\\'; *str++ = 'b'; break;
      /*
      case '\a': *str++ = '\\'; *str++ = 'a'; break;
      */
      default:
        if (isprint(*data)) {
          *str++ = *data;
        }
        else {
          *str++ = '\\';
          *str++ = octal[(*data >> 6) & 7];
          *str++ = octal[(*data >> 3) & 7];
          *str++ = octal[(*data     ) & 7];
        }
    }
    data++;
  }
  *str++ = '"';
  *str = 0;
  return static_cast<int>(str - aStr);
}

bool strParse(const char*& aStr, char* aData, int& aDataSize) {
  char* data = aData;
  const char* str = aStr;
  bool result = true;
  while (isspace(*str)) {
    str++;
  }
  if (*str != '"') {
    while (*str && !isspace(*str) && (*str != ',')) {
      *data++ = *str++;
    }
  }
  else {
    str++;
    for (;;) {
      if (*str == '"') {
        str++;
        break;
      }
      if (*str == 0) {
        result = false;
        break;
      }
      if (*str == '\\') {
        str++;
        int code;
        int digit;
        switch (*str) {
          case 'r': *data++ = '\r'; str++; break;
          case 'n': *data++ = '\n'; str++; break;
          case 't': *data++ = '\t'; str++; break;
          case 'v': *data++ = '\v'; str++; break;
          case 'b': *data++ = '\b'; str++; break;
          case 'a': *data++ = '\a'; str++; break;
          case '\'':
          case '\"':
          case '\\': *data++ = *str++; break;
          case 'x':
            str++;
            if (!hexDigit(*str++, code)) {
              result = false;
              break;
            }
            for ( ; hexDigit(*str, digit); str++) {
              code = (code * 16) + digit;
              if (code > UCHAR_MAX) {
                result = false;
        	break;
              }
            }
            *data++ = code;
            break;
          default:
            if (!octalDigit(*str++, code)) {
              result = false;
              break;
            }
            for ( ; octalDigit(*str, digit); str++) {
              code = (code * 8) + digit;
              if (code > UCHAR_MAX) {
                result = false;
        	break;
              }
            }
            *data++ = code;
        }
        break;
      }
      else {
        *data++ = *str++;
      }
    }
  }
  aDataSize = static_cast<int>(data - aData);
  aStr = str;
  return result;
}

bool strToInt(const char* aStr, int aMin, int aMax, int& aResult) {
  char* end;
  aResult = strtol(aStr, &end, 0);
  return (end != aStr) && (aResult >= aMin) && (aResult <= aMax);
}

bool strToInt(const char* aStr, int& aResult) {
  return strToInt(aStr, LONG_MIN, LONG_MAX, aResult);
}

bool strToBool(const char* aStr, bool& aResult) {
  while (isspace(*aStr)) {
    aStr++;
  }
  switch (*aStr) {
    case 'y': case 'Y':
    case 't': case 'T':
    case '1':
      aResult = true;
      return true;
    case 'n': case 'N':
    case 'f': case 'F':
    case '0':
      aResult = false;
      return true;
  }
  return false;
}

// Probably looks stupid but:
// o  does not assume anything about the character encoding;
// o  is efficient (a smart compiller should generate a jump table);
// o  is readable.

bool octalDigit(char aChar, int& aDigit) {
  switch (aChar) {
    case '0': aDigit = 0; return true;
    case '1': aDigit = 1; return true;
    case '2': aDigit = 2; return true;
    case '3': aDigit = 3; return true;
    case '4': aDigit = 4; return true;
    case '5': aDigit = 5; return true;
    case '6': aDigit = 6; return true;
    case '7': aDigit = 7; return true;
    default : return false;
  }
}

bool decimalDigit(char aChar, int& aDigit) {
  switch (aChar) {
    case '0': aDigit = 0; return true;
    case '1': aDigit = 1; return true;
    case '2': aDigit = 2; return true;
    case '3': aDigit = 3; return true;
    case '4': aDigit = 4; return true;
    case '5': aDigit = 5; return true;
    case '6': aDigit = 6; return true;
    case '7': aDigit = 7; return true;
    case '8': aDigit = 8; return true;
    case '9': aDigit = 9; return true;
    default : return false;
  }
}

bool hexDigit(char aChar, int& aDigit) {
  if (decimalDigit(aChar, aDigit)) return true;
  switch (aChar) {
    case 'a': aDigit = 10; return true;
    case 'b': aDigit = 11; return true;
    case 'c': aDigit = 12; return true;
    case 'd': aDigit = 13; return true;
    case 'e': aDigit = 14; return true;
    case 'f': aDigit = 15; return true;
  }
  switch (aChar) { // more chances to be compiled into jump tables
    case 'A': aDigit = 10; return true;
    case 'B': aDigit = 11; return true;
    case 'C': aDigit = 12; return true;
    case 'D': aDigit = 13; return true;
    case 'E': aDigit = 14; return true;
    case 'F': aDigit = 15; return true;
    default : return false;
  }
}

